from django.contrib import admin
from django import forms
from django.utils.translation import ugettext_lazy as _
from django.db.models import get_model
from django.db import transaction
from django.forms.util import ErrorList
from django.contrib.admin import widgets
from django.contrib.contenttypes.models import ContentType

from jumbo.admin.options import ExtendibleModelAdmin
from jumbo.core.urlresolvers import reverse
from jumbo.forms import DateWidget
from jumbo.apps.autocomplete import AutocompleteInlineGFKCharField, autocomplete

from agenda.models import Appointment, Role, Membership, AppointmentType
from agenda import views as app_views

from jumbo.middleware.thread_local import get_request


#######################################
# MEMBERSHIP                          #
#######################################
autocomplete.register(
    'obj_member', views = 'agenda.ajax.get_objmember', limit=10
)

class MemberShipSearchForm(forms.ModelForm):
    class Meta:
        model = Membership
        display_fields = (('appointment__title__icontains','invite_reason__icontains'), 
                          ('content_object__icontains','status'),
                          ('date_begin__lte', 'date_end__gte'),)

    def __init__(self, *args, **kws):
        super(MemberShipSearchForm, self).__init__(*args, **kws)
        self.fields['invite_reason__icontains'] = forms.CharField(label=_("Invite Reason"), required=False)
        self.fields['appointment__title__icontains'] = forms.CharField(label=_("Appuntamento"), required=False)
        self.fields['content_object__icontains'] = forms.CharField(label=_("Content Object"), required=False)
        self.fields['date_begin__lte'] = forms.DateField(label=_("Data di inizio"), required=False, widget=DateWidget)
        self.fields['date_end__gte'] = forms.DateField(label=_("Data di fine"), required=False, widget=DateWidget)


class MemberShipForm(forms.ModelForm):

    obj_member = AutocompleteInlineGFKCharField('obj_member', label=_("Object"))

    class Meta:
        model = Membership


    def __init__(self, *args, **kws):
        super(MemberShipForm, self).__init__(*args, **kws)

        if kws.has_key('instance'):
            ct = str(self.instance.content_type.id) 
            obj = str(self.instance.object_id)
            self.initial['obj_member'] = "%s-%s" %(ct, obj)


    def save(self, commit=False):
        if self.instance.id:
            ## UPDATE
            m = self.instance
        else :
            ## INSERT
            m = Membership()

        o = self.cleaned_data['obj_member']
        m.appointment_id = self.cleaned_data['appointment'].id

        m.status = self.cleaned_data['status']
        m.role = self.cleaned_data['role']
        m.invite_reason = self.cleaned_data['invite_reason']

        list = o.split('-')
        m.content_type_id = list[0]
        m.object_id = list[1]
        m.save()
        return m


class MembershipAdmin(ExtendibleModelAdmin):
    list_display = ('appointment', 
                    'status_icon', 
                    'date_begin',
                    'date_end',
                    'content_object', 
                    'invite_reason',
                    'get_edit_icon',
                    'get_delete_icon')
    default_group_by = 'appointment'
    list_filter = ('appointment',)
    fieldsets = (
        (None, {
            'fields': (
                       ('content_type',),
                       ('object_id',),
                       ('appointment',),
                       ('role',),
                       ('status'),
                       )
        }),
    )
    form = MemberShipForm
    search_form = MemberShipSearchForm


    def date_begin(self,obj):
        return obj.appointment.date_begin.strftime('%d %b %Y')


    def date_end(self,obj):
        return obj.appointment.date_end.strftime('%d %b %Y')


    def status_icon(self,obj):
        if obj.status == "A":
            return "Assente"
        elif obj.status == "O":
            return "Occupato"
        elif obj.status == "P":
            return "Presente"
        else: 
            return ""
    status_icon.short_description = "Stato"


#######################################
# INLINES                             #
#######################################

class MembershipInline(admin.TabularInline):
   
    model = Membership
    extra = 0
    fieldsets = (
        (None, {
            'fields': (
                       ('obj_member','role', 'status','invite_reason')
                       )
        }), 
    )
    exclude = ['content_type', 'object_id']
    form = MemberShipForm
    template = "admin/agenda/appointment/membership_inline.html" 

#######################################
# APPUNTAMENTO                        #
#######################################

class AppointmentSearchForm(forms.ModelForm):
    class Meta:
        model = Appointment
        display_fields = (('date_begin__gte','date_end__lte'), 
                          ('title__icontains','description__icontains'),)

    def __init__(self, *args, **kws):
        super(AppointmentSearchForm, self).__init__(*args, **kws)
        self.fields['description__icontains'] = forms.CharField(label=_("Descrizione"), required=False)
        self.fields['title__icontains'] = forms.CharField(label=_("Titolo"), required=False)
        self.fields['date_begin__gte'] = forms.DateField(label=_("Data di inizio"), required=False, widget=DateWidget)
        self.fields['date_end__lte'] = forms.DateField(label=_("Data di fine"), required=False, widget=DateWidget)


class AppointmentForm(forms.ModelForm):

    class Meta:
        model = Appointment
#        display_fields = (('periodicity',),)


#    def __init__(self, *args, **kws):
#        super(AppointmentForm, self).__init__(*args, **kws)
#        self.fields['periodicity'].label=_("Periodicita'")


    def clean(self):
        db = self.cleaned_data.get('date_begin')
        de = self.cleaned_data.get('date_end')
        if de < db:
            self._errors["date_end"] = ErrorList(["La data di fine non puo' essere antecedente alla data di inizio!"])
            del self.cleaned_data['date_end']
        tb = self.cleaned_data.get('time_begin')
        te = self.cleaned_data.get('time_end')
        if te < tb:
             self._errors["time_end"] = ErrorList(["L'ora di fine non puo' essere antecedente all'ora di inizio!"])   
             del self.cleaned_data['time_end']
        return self.cleaned_data 


class AppointmentAdmin(ExtendibleModelAdmin):

    list_display = ('title', 'description',
                    'date_begin','date_end','time_begin','time_end', 'visibility',
                    'get_detail_icon', 'get_edit_icon','get_delete_icon')
    form = AppointmentForm
    default_group_by = '-date_begin'
    ordering = ['time_begin']
    fieldsets = (
        (_('Additional information'), {
            'classes': ('collapse',),
            'fields': ((
                'date_create', 'date_last_modify','creator','last_modifier',
            ))
        }),                 
        (None, {
            'fields': (
                       ('title',),
                       ('description',),
                       ('date_begin','date_end'),
                       ('time_begin','time_end'),
                       ('visibility','tipology'),
                       )
        }),
    )
    inlines = [MembershipInline]
    search_form = AppointmentSearchForm


    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(AppointmentAdmin, self).get_urls()
        my_urls = patterns('',

            url(r'^calendar/$'                        , 'jumbo.apps.calendar.views.calendar_list'
                                                      , {
                                                            'model' : Appointment,
                                                            'field_date_begin': 'date_begin',
                                                            'field_date_end': 'date_end',
                                                            'template_dir':'admin/agenda/appointment/calendar'
                                                        }
                                                      , name='agenda_appointment_calendar'),

            url(r'^calendar/(?P<type_list>\w+)/$'     , 'jumbo.apps.calendar.views.calendar_list'
                                                      , {
                                                            'model' : Appointment,
                                                            'field_date_begin': 'date_begin',
                                                            'field_date_end': 'date_end',
                                                            'template_dir':'admin/agenda/appointment/calendar'
                                                        }
                                                      , name='agenda_appointment_calendar'),

            url(r'^calendar/(?P<type_list>\w+)/(?P<day>\d+)/$', 'jumbo.apps.calendar.views.calendar_list'
                                                              , {
                                                                    'model' : Appointment,
                                                                    'field_date_begin': 'date_begin',
                                                                    'field_date_end': 'date_end',
                                                                    'template_dir':'admin/agenda/appointment/calendar'
                                                                }
                                                              , name='agenda_appointment_calendar'),
            )
        return my_urls + urls 

    def PeriodicityWithLabel(self,obj):
        return obj.get_periodicity_display
    PeriodicityWithLabel.short_description = "Periodicita'" 


#######################################
# ROLE                                 #
#######################################
               
class RoleForm(forms.ModelForm):
    class Meta:
        model = Role
        
    def __init__(self, *args, **kws):
        super(RoleForm, self).__init__(*args, **kws)
        self.fields['date_begin'] = forms.DateField(label=_("Data inizio"), required=False, widget=DateWidget)

class RoleAdmin(ExtendibleModelAdmin):
    list_display = ('name',
                    'get_edit_icon','get_delete_icon')
    form = RoleForm
    search_fields = ['name']
    fieldsets = (
        (None, {
            'fields': (
                       ('name',),
                       )
        }),
        ('Additional information', {
             'classes': ('collapse',),
             'fields': (
                        ('date_create', 'date_last_modify','creator','last_modifier',)
                        )
        }),
    )
    
#######################################
# APPOINTMENT TYPE                    #
#######################################


class AppointmentTypeAdmin(ExtendibleModelAdmin):
    list_display = ('name',
                    'get_edit_icon','get_delete_icon')
    search_fields = ['name']
    fieldsets = (
        (None, {
            'fields': (
                       ('name',),
                       )
        }),
        ('Additional information', {
             'classes': ('collapse',),
             'fields': (
                        ('date_create', 'date_last_modify','creator','last_modifier',)
                        )
        }),
    )

admin.site.register(Appointment,AppointmentAdmin)
admin.site.register(Role,RoleAdmin)
admin.site.register(Membership,MembershipAdmin)
admin.site.register(AppointmentType,AppointmentTypeAdmin)
