# -*- coding: utf-8 -*-
from datetime import datetime, date, time
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jumbo.core.urlresolvers import reverse
from jumbo.middleware.thread_local import get_current_user

#######################################
# APPUNTAMENTO                        #
#######################################
PERIODICITY = (
    ('d', _('Tutti i Giorni')),
    ('w', _('Tutte le Settimane')),
    ('m', _('Tutti i Mesi')),
    ('y', _('Tutti gli Anni')),
)
VISIBILITY = (
    ('p', _('Pubblico')),
    ('r', _('Privato')),
    ('o', _('Occupato')),
)


#######################################
# TIPI                                #
#######################################
    
class AppointmentType(DateModel, UserModel, OrderedModel):
    ## Campi Modello
    name = models.CharField (
        max_length = 100,
        db_index = True,
        verbose_name = _('Name'),
    )

    class Meta:
        verbose_name = _("Tipologia")
        verbose_name_plural = _("Tipologie")

    def __unicode__(self):
        return u"%s" % self.name


class UserAppointmentsManager(models.Manager):

    def get_query_set(self):
        user = get_current_user()
        qs = super(UserAppointmentsManager, self).get_query_set()
        a = Appointment()

        #if a.
#        if user.has_perm('agenda.list_all_appointments'):
#            return qs
#        qs = qs.filter()

        return qs
        
#        if user.has_perm("person.list_all_appointments"):
#            return qs
#        if user.has_perm("person.view_appointment"):
#            c = ContentType.objects.get_for_model(Person)
#            ms = Membership.objects.filter(content_type=c,object_id=user.contact.id)
#            q = Q()
#            for a in ms:
#                q = Q(pk=a.appointment.id)
#                q = q | q1      
#            return qs.filter(q)

class Appointment(DateModel, UserModel, OrderedModel):
    objects = UserAppointmentsManager()

    origin_type = models.ForeignKey(
        ContentType,
        blank=True, null=True, editable=False
    )
    origin_id = models.PositiveIntegerField(
        null=True, editable=False
    )
    origin = generic.GenericForeignKey('origin_type', 'origin_id')
 
    assigned_by = models.ForeignKey(
        User,
        blank = True, null = True,
        related_name = "appointments_assigned_by_me",
        verbose_name = _("Assegnato da"),
    )

    tipology = models.ForeignKey(
        AppointmentType,
        blank = True, null = True,
        related_name = "appointments",
        verbose_name = _("Tipologia Appuntamento"),
    )
    
    dayperiodicity = models.ForeignKey(
        "DayPeriodicity",
        blank = True, null = True,
        related_name = "appointments",
        verbose_name = _("Appuntamenti Giornalieri")
    )
    weekperiodicity = models.ForeignKey(
        "WeekPeriodicity",
        blank = True, null = True,
        related_name = "appointments",
        verbose_name = _("Appuntamenti Settimanali")
    )
    monthperiodicity = models.ForeignKey(
        "MonthPeriodicity",
        blank = True, null = True,
        related_name = "appointments",
        verbose_name = _("Appuntamenti Mensili")
    )
    yearperiodicity = models.ForeignKey(
        "YearPeriodicity",
        blank = True, null = True,
        related_name = "appointments",
        verbose_name = _("Appuntamenti Annuali")
    )
    ## Campi Modello
    title = models.CharField(
        max_length = 100,
        verbose_name = _("Titolo"),
    )
    description = models.TextField(
        blank = True, null = True,
        verbose_name = _("Descrizione"),
    )
    date_begin = models.DateField(
        db_index = True,
        verbose_name = _("Data Inizio"),
    )
    date_end = models.DateField(
        db_index = True,
        verbose_name = _("Data Fine"),
    )
    time_begin = models.TimeField(
        db_index = True,
        verbose_name = _("Ora Inizio"),
    )
    time_end = models.TimeField(
        db_index = True,
        verbose_name = _("Ora Fine"),
    )
    periodicity = models.CharField(
        max_length = 1,
        blank = True, null = True,
        choices = PERIODICITY,
        db_index = True,
        verbose_name = _("Periodicita'"),
    )
    visibility = models.CharField(
        max_length = 1,
        blank = True, null = True,
        choices = VISIBILITY,
        db_index = True,
        verbose_name = _("Visibilita'"),
    )

    class Meta:
        verbose_name = _("Appuntamento")
        verbose_name_plural = _("Appuntamenti")
        permissions = (
            ('view_appointment', 'Can View Appointments'),
            ('list_all_appointments', 'Can List All Appointments'),
        )

    def __unicode__(self):
        return unicode('%s: %s - %s %s - %s') % (
            self.title,
            self.date_begin.__str__(), self.date_end.__str__(),
            self.time_begin, self.time_end)

    def get_link_origin(self):
        if self.origin:
            link = reverse('admin:%s_%s_changelist' % (
                self.origin_type.app_label, self.origin_type.model
            ))
            return "%s?id=%s" % (link, self.origin_id)
        return ""

    def print_html_calendar(self):
        return """
            <b><a href="%s">%s</a></b><br/>
            %s - %s
        """ % (
            reverse("admin:%(app)s_%(module)s_detail" % {
                'app':self._meta.app_label,
                'module':self._meta.module_name }, self.id), 
            self.title,
            self.time_begin.strftime('%H:%M'), self.time_end.strftime('%H:%M')
        )

    def print_xml_calendar(self):
        return """
            <text:p>%s - %s</text:p><text:p>%s</text:p>
        """ % (
            self.time_begin.strftime('%H:%M'), self.time_end.strftime('%H:%M'),
            self.title
        )

    def modifica_appuntamento(self):
        self.save()

    def modifica_serie_appuntamenti(self):
        if periodicity == "d":
            serie = self.dayperiodicity
            date_diff = self.date_end - self.date_begin
            apps = serie.appointments.exclude(pk=self.id)
            for app in apps:
                old_id = app.id
                old_date_begin = app.date_begin
                old_date_end = app.date_begin + date_diff
                app = self
                app.id = old_id
                app.date_begin = old_date_begin
                app.date_end = old_date_end
                app.save()

    def modifica_serie_appuntamenti_precedenti(self):
        pass

    def modifica_serie_appuntamenti_successivi(self):
        pass

    def cancella_appuntamento(self):
        self.delete()

    def cancella_serie_appuntamenti(self):
        pass

    def cancella_serie_appuntamenti_precedenti(self):
        pass

    def cancella_serie_appuntamenti_successivi(self):
        pass

    def get_members_list(self):
        #recupero gli oggetti membri di quell'appuntamento
        m = Membership.objects.filter(appointment=self.id)
        return m

class DayPeriodicity(DateModel, UserModel):
    ## Campi Modello
    title = models.CharField(
        max_length = 100,
        verbose_name = _("Titolo"),
    )
    description = models.TextField(
        blank = True, null = True,
        verbose_name = _("Description"),
    )
    date_begin = models.DateField(
        db_index = True,
        verbose_name = _("Data Inizio"),
    )
    date_end = models.DateField(
        db_index = True,
        verbose_name = _("Data Fine"),
    )
    time_begin = models.TimeField(
        db_index = True,
        verbose_name = _("Ora Inizio"),
    )
    time_end = models.TimeField(
        db_index = True,
        verbose_name = _("Ora Fine"),
    )
    repeat_days = models.IntegerField(
        verbose_name = _("Repeat Every Days"),
    )

    class Meta:
        verbose_name = _("Serie Giorno")
        verbose_name_plural = _("Serie Giorno")

    def __unicode__(self):
        return unicode('%s' % self.title)

class WeekPeriodicity(DateModel, UserModel):
    ## Campi Modello
    title = models.CharField(
        max_length = 100,
        verbose_name = _("Titolo"),
    )
    description = models.TextField(
        blank = True, null = True,
        verbose_name = _("Description"),
    )
    date_begin = models.DateField(
        db_index = True,
        verbose_name = _("Data Inizio"),
    )
    date_end = models.DateField(
        db_index = True,
        verbose_name = _("Data Fine"),
    )
    time_begin = models.TimeField(
        db_index = True,
        verbose_name = _("Ora Inizio"),
    )
    time_end = models.TimeField(
        db_index = True,
        verbose_name = _("Ora Fine"),
    )
    repeat_weeks = models.IntegerField(
        verbose_name = _("Repeat Every Weeks"),
    )
    mon = models.BooleanField(
        verbose_name = _("Monday"),
    )
    tue = models.BooleanField(
        verbose_name = _("Tuesday"),
    )
    wed = models.BooleanField(
        verbose_name = _("Wednesday"),
    )
    thu = models.BooleanField(
        verbose_name = _("Thursday"),
    )
    fri = models.BooleanField(
        verbose_name = _("Friday"),
    )
    sat = models.BooleanField(
        verbose_name = _("Saturday"),
    )
    sun = models.BooleanField(
        verbose_name = _("Sunday"),
    )

    class Meta:
        verbose_name = _("Serie Settimana")
        verbose_name_plural = _("Serie Settimana")

    def __unicode__(self):
        return unicode('%s' %self.title)

TYPE_DAY = (
    ("w", _("Week Day"),),
    ("m", _("Month Day"),),
)

class MonthPeriodicity(DateModel, UserModel):
    ## Campi Modello
    title = models.CharField(
        max_length = 100,
        verbose_name = _("Titolo"),
    )
    description = models.TextField(
        blank = True, null = True,
        verbose_name = _("Description"),
    )
    date_begin = models.DateField(
        db_index = True,
        verbose_name = _("Data Inizio"),
    )
    date_end = models.DateField(
        db_index = True,
        verbose_name = _("Data Fine"),
    )
    time_begin = models.TimeField(
        db_index = True,
        verbose_name = _("Ora Inizio"),
    )
    time_end = models.TimeField(
        db_index = True,
        verbose_name = _("Ora Fine"),
    )
    repeat_months = models.IntegerField(
        verbose_name = _("Repeat Every Months"),
    )
    type_day = models.CharField(
        choices = TYPE_DAY,
        max_length = 1,
        db_index = True,
        verbose_name = _("Week Day"),
    )

    class Meta:
        verbose_name = _("Serie Mese")
        verbose_name_plural = _("Serie Mese")

    def __unicode__(self):
        return unicode('%s' % self.title)

class YearPeriodicity(DateModel, UserModel):
    ## Campi Modello
    title = models.CharField(
        max_length = 100,
        verbose_name = _("Titolo"),
    )
    description = models.TextField(
        blank = True, null = True,
        verbose_name = _("Description"),
    )
    date_begin = models.DateField(
        db_index = True,
        verbose_name = _("Data Inizio"),
    )
    date_end = models.DateField(
        db_index = True,
        verbose_name = _("Data Fine"),
    )
    time_begin = models.TimeField(
        db_index = True,
        verbose_name = _("Ora Inizio"),
    )
    time_end = models.TimeField(
        db_index = True,
        verbose_name = _("Ora Fine"),
    )
    repeat_years = models.IntegerField(
        verbose_name = _("Repeat Every Years"),
    )

    class Meta:
        verbose_name = _("Serie Anno")
        verbose_name_plural = _("Serie Anno")

    def __unicode__(self):
        return unicode('%s' % self.title)

#######################################
# RUOLO                               #
#######################################
class Role(DateModel, UserModel, OrderedModel):
    ## Campi Modello
    name = models.CharField (
        max_length = 100,
        db_index = True,
        verbose_name = _('Name'),
    )

    class Meta:
        verbose_name = _("Role")
        verbose_name_plural = _("Roles")

    def __unicode__(self):
        return u"%s" % self.name

#######################################
# INVITO                              #
#######################################
STATUS_MEM = (
    ('A', _('ASSENTE')),
    ('O', _('OCCUPATO')),
    ('P', _('PRESENTE')),
)

class Membership(models.Model):
    ## ForeignKey
    # Content-object field
    content_type   = models.ForeignKey(ContentType,
            verbose_name=_('content type'),
            related_name="content_type_set_for_%(class)s")
    object_id      = models.TextField(_('object ID'))
    content_object = generic.GenericForeignKey()

    appointment = models.ForeignKey(
        Appointment,
        db_index = True,
    )
    role = models.ForeignKey(
        Role,
        db_index = True,
        blank = True, null = True,
    )
    ## Campi Modello
    status = models.CharField (
        max_length = 1,
        blank = True, null = True,
        db_index = True,
        choices = STATUS_MEM,
        verbose_name = _("Status"),
    )
    invite_reason = models.CharField(
        max_length = 100,
        blank = True, null = True,
        verbose_name = _("Invite Reason"),
    )

    class Meta:
        verbose_name = _("Member Ship")
        verbose_name_plural = _("Member Ships")

    def __unicode__(self):
        return u"%s: %s - %s" % (
            self.appointment.title, 
            self.appointment.date_begin.__str__(), 
            self.appointment.date_begin.__str__())
 
